<?php
/*
@package wp-solar
@internal  Create graphs
@since 0.4
@todo nothing
*/

if ( ! defined( 'ABSPATH' ) ) { 
    exit;
}


/* ----------------------------------------------- */

function wp_solar_power_graph() {
	global $wpdb;

	$power_units = get_option( 'wp_solar_power_display_units' );
	$power_divider = wp_solar_get_power_divider();
	$wp_solar_power_display_color = get_option( 'wp_solar_power_display_color' );
	$table_name = $wpdb->prefix . 'wp_solar_production';
	$sql = "SELECT date, sum(power) AS power FROM $table_name GROUP BY date ORDER BY date ASC;";
	$results = $wpdb->get_results( $sql );
	$wp_solar_production_data = '';
	$wp_solar_production_label = '';
	foreach ( $results as $result ) 
	{
		$date = substr($result->date, 0, -12);
		$wp_solar_production_label .= '"'.$date.'",';
		$wp_solar_production_data .= $result->power / $power_divider.',';
	}
	$wp_solar_production_label = rtrim($wp_solar_production_label, ",");
	$wp_solar_production_data= rtrim($wp_solar_production_data, ",");
	$production_title = __('Production', 'wp-solar');
	$wp_solar_power_graph = '<canvas id="wp_solar_power_graph" width="400" height="250"></canvas>';
	$wp_solar_power_graph .= '<script>
	var ctx = document.getElementById("wp_solar_power_graph");
	var wp_solar_power_graph = new Chart(ctx,{
		type: "bar",
		data:{
			labels:['. $wp_solar_production_label .'],
			datasets:[{
				label:"'. $production_title .' ('.$power_units.')",
				data:['. $wp_solar_production_data .'],
				backgroundColor:"'. $wp_solar_power_display_color .'"
				}]
			},
		options:{
			scales:{
				xAxes:[{
				}],
				yAxes:[{
					ticks:{
						min: 0
					}
				}]
			}
		}
	});
	</script>';

	set_transient( 'wp_solar_power_graph', $wp_solar_power_graph, DAY_IN_SECONDS );
}

/* ----------------------------------------------- */

function wp_solar_annual_power_graph() {
	global $wpdb;

	$table_name = $wpdb->prefix . 'wp_solar_production';
	$production_title = __('Production', 'wp-solar');
	$power_units = get_option( 'wp_solar_power_display_units' );
	$power_divider = wp_solar_get_power_divider();
	$wp_solar_annual_power_display_colors = get_option( 'wp_solar_annual_power_display_colors' );
	$colors = preg_split ("/\,/", $wp_solar_annual_power_display_colors);
	$number_of_colors = count($colors);
	$sql = "SELECT date FROM $table_name ORDER BY date ASC LIMIT 0,1;";
	$row = $wpdb->get_row( $sql );
	$beginning_year = substr($row->date, 0, -15);
	$sql = "SELECT date FROM $table_name ORDER BY date DESC LIMIT 0,1;";
	$row = $wpdb->get_row( $sql );
	$ending_year = substr($row->date, 0, -15);
	$wp_solar_annual_power_graph = '<canvas id="wp_solar_annual_power_graph" width="400" height="250"></canvas>';
	$wp_solar_annual_power_graph .= '<script>
	var ctx = document.getElementById("wp_solar_annual_power_graph");
	var wp_solar_annual_power_graph = new Chart(ctx,{
		type: "bar",
		data:{labels:["01","02","03","04","05","06","07","08","09","10","11","12"],';
	$wp_solar_annual_power_graph .= 'datasets:[';
	$i = 0;
	for ( $production_year = $beginning_year; $production_year <= $ending_year; $production_year++ ) {
		$dataset_label = $production_title .' '. $production_year .' ('.$power_units.')';
		$dataset_data = "";
		for ( $production_month = 01; $production_month <= 12; $production_month++ ) {
				$formatted_production_month = sprintf("%02d", $production_month);
				$production_period = $production_year.'-'.$formatted_production_month.'-01 00:00:00';
				$sql = "SELECT sum(power) AS power FROM $table_name WHERE date = '$production_period';";
				$result = $wpdb->get_row( $sql );
				$power = $result->power / $power_divider;
				$dataset_data .= "$power,";
		}
		$dataset_data= rtrim($dataset_data, ",");
		$wp_solar_annual_power_graph .= '{label:"'.$dataset_label.'",data:['.$dataset_data.'],backgroundColor:"'. $colors[$i % $number_of_colors] .'"},';
		$i++;
	}
	$wp_solar_annual_power_graph .= ']';
	$wp_solar_annual_power_graph .= '			},
		options:{
			scales:{
				xAxes:[{
				}],
				yAxes:[{
					ticks:{
						min: 0
					}
				}]
			}
		}
	});
	</script>';
	
	set_transient( 'wp_solar_annual_power_graph', $wp_solar_annual_power_graph, DAY_IN_SECONDS );
}

/* ----------------------------------------------- */

function wp_solar_annual_power_graph_per_site() {
	global $wpdb;

	$table_name = $wpdb->prefix . 'wp_solar_production';
	$production_title = __('Production', 'wp-solar');
	$power_units = get_option( 'wp_solar_power_display_units' );
	$power_divider = wp_solar_get_power_divider();
	$wp_solar_annual_power_display_colors = get_option( 'wp_solar_annual_power_display_colors' );
	$colors = preg_split ("/\,/", $wp_solar_annual_power_display_colors);
	$number_of_colors = count($colors);
	$sql = "SELECT date FROM $table_name ORDER BY date ASC LIMIT 0,1;";
	$row = $wpdb->get_row( $sql );
	$beginning_year = substr($row->date, 0, -15);
	$sql = "SELECT date FROM $table_name ORDER BY date DESC LIMIT 0,1;";
	$row = $wpdb->get_row( $sql );
	$ending_year = substr($row->date, 0, -15);
	$wp_solar_annual_power_graph_per_site = '<canvas id="wp_solar_annual_power_graph_per_site" width="400" height="250"></canvas>';
	$wp_solar_annual_power_graph_per_site .= '<script>
	var ctx = document.getElementById("wp_solar_annual_power_graph_per_site");
	var wp_solar_annual_power_graph_per_site = new Chart(ctx,{
		type: "bar",
		data:{labels:["01","02","03","04","05","06","07","08","09","10","11","12"],';
	$wp_solar_annual_power_graph_per_site .= 'datasets:[';
	$i = 0;
	for ( $production_year = $beginning_year; $production_year <= $ending_year-1; $production_year++ ) {
		$dataset_label = $production_title .' '. $production_year .' ('.$power_units.')';
		$dataset_data = "";
		for ( $production_month = 01; $production_month <= 12; $production_month++ ) {
				$formatted_production_month = sprintf("%02d", $production_month);
				$production_period = $production_year.'-'.$formatted_production_month.'-01 00:00:00';
				$sql = "SELECT sum(power) AS power FROM $table_name WHERE date = '$production_period';";
				$result = $wpdb->get_row( $sql );
				$power = $result->power / $power_divider;
				$dataset_data .= "$power,";
		}
		$dataset_data= rtrim($dataset_data, ",");
		$wp_solar_annual_power_graph_per_site .= '{label:"'.$dataset_label.'",data:['.$dataset_data.'],type:"line"},';
		$i++;
	}
	$dataset_label = $production_title .' '. $ending_year .' ('.$power_units.')';
	$dataset_data = "";
	for ( $production_month = 01; $production_month <= 12; $production_month++ ) {
			$formatted_production_month = sprintf("%02d", $production_month);
			$production_period = $ending_year.'-'.$formatted_production_month.'-01 00:00:00';
			$sql = "SELECT sum(power) AS power FROM $table_name WHERE date = '$production_period';";
			$result = $wpdb->get_row( $sql );
			$power = $result->power / $power_divider;
			$dataset_data .= "$power,";
	}
	$dataset_data= rtrim($dataset_data, ",");
	$wp_solar_annual_power_graph_per_site .= '{label:"'.$dataset_label.'",data:['.$dataset_data.'],backgroundColor:"blue"}';	$wp_solar_annual_power_graph_per_site .= ']';
	$wp_solar_annual_power_graph_per_site .= '			},
		options:{
			scales:{
				xAxes:[{
				}],
				yAxes:[{
					ticks:{
						min: 0
					}
				}]
			}
		}
	});
	</script>';
	
	set_transient( 'wp_solar_annual_power_graph_per_site', $wp_solar_annual_power_graph_per_site, DAY_IN_SECONDS );
}