<?php
/*
Plugin Name: WP Solar
Description: Tools for displaying solar panel data on web sites
Tags: solar
Version: 0.5.2.1
Author: Carl Conrad
Author URI: https://carlconrad.net
License: GPL2
Text Domain: wp-solar

@package wp-solar
@internal Main file
@since 0.1
@todo nothing
*/

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

// add_action( 'init', 'wp_solar_total_power_block_register_block' );

if ( is_admin() ){
	require_once ('include/settings.php');
	add_action( 'admin_menu', 'wp_solar_options_page' );
	add_action( 'admin_init', 'wp_solar_register_settings' );
}
else {
	add_action( 'wp_enqueue_scripts', 'wp_solar_load_scripts' );
	require_once ('include/functions.php');
	require_once ('include/shortcodes.php');
}

register_activation_hook( __FILE__, 'wp_solar_cronjob_activate' );

register_deactivation_hook(__FILE__, 'wp_solar_cronjob_deactivate');

add_action ('wp_solar_cronjob', 'wp_solar_cronjob_task');

/* ----------------------------------------------- */

function wp_solar_total_power_block_register_block() {
    wp_register_script(
        'total-power-block',
        plugins_url( 'total-power-block.js', __FILE__ ),
        array( 'wp-blocks', 'wp-i18n', 'wp-element' )
    );
 
    register_block_type( 'wp-solar/total-power-block', array(
        'editor_script' => 'total-power-block',
    ) );
 
}

/* ----------------------------------------------- */

function wp_solar_load_scripts() {
	wp_enqueue_script ('chart', plugins_url('assets/Chart.js/Chart.min.js', __FILE__),false, false ,false);
}

/* ----------------------------------------------- */

function wp_solar_cronjob_activate() {
	if( !wp_next_scheduled( 'wp_solar_cronjob' ) ) {
		wp_schedule_event( time(), 'hourly', 'wp_solar_cronjob' );
	}
}

function wp_solar_cronjob_deactivate() {
	wp_clear_scheduled_hook('wp_solar_cronjob');
}

function wp_solar_cronjob_task() {
	global $wpdb;
	 
	if ( $wp_solar_solaredge_api_keys = get_option( 'wp_solar_solaredge_api_keys' ) ) {

		$today = date('Y-m-d');
		$wp_solar_production = 0;
		$charset_collate = $wpdb->get_charset_collate();

		$sites_table_name = $wpdb->prefix . 'wp_solar_sites';
		$sql_drop_sites = "DROP TABLE IF EXISTS $sites_table_name;";
		$sql_create_sites = "CREATE TABLE IF NOT EXISTS $sites_table_name (
			site VARCHAR(50),
			name VARCHAR(50)
		) ENGINE=InnoDB $charset_collate;";
		$sql_alter_sites = "ALTER TABLE $sites_table_name ADD UNIQUE INDEX(site);";

		$production_table_name = $wpdb->prefix . 'wp_solar_production';
		$sql_drop_production = "DROP TABLE IF EXISTS $production_table_name;";
		$sql_create_production = "CREATE TABLE IF NOT EXISTS $production_table_name (
			api VARCHAR(50),
			site VARCHAR(50),
			date DATETIME,
			power INT(10) NOT NULL
		) ENGINE=InnoDB $charset_collate;";
		$sql_alter_production = "ALTER TABLE $production_table_name ADD INDEX(date);";

		$sql_insert_sites = "INSERT INTO $sites_table_name (site, name) VALUES ";
//		$sql_insert_sites = array();
		$sql_insert_production = "INSERT INTO $production_table_name (api, site, date, power) VALUES ";
		foreach(preg_split("/((\r?\n)|(\r\n?))/", $wp_solar_solaredge_api_keys) as $line){
			$data = explode(',', $line);
			$api_call = 'https://monitoringapi.solaredge.com/site/'. $data[1] .'/details.json?api_key='. $data[0];
			$response = wp_remote_get( $api_call , array(
				'method' => 'GET',
				'timeout' => 30,
				'redirection' => 5,
				'httpversion' => '1.0',
				'blocking' => true,
				'headers' => array('Content-type' => 'application/json')
				)
			);
			$body = json_decode( $response['body'] );
			$startDate = $body->details->installationDate;
			$name = $body->details->name;
			$sql_insert_sites .= "('$data[1]','$name'),";
//			$sql_insert_sites[] = array("site" => $data[1], "name" => $name);
			$api_call = 'https://monitoringapi.solaredge.com/site/'. $data[1] .'/energy?timeUnit=MONTH&startDate='. $startDate .'&endDate='. $today .'&api_key='. $data[0];
			$response = wp_remote_get( $api_call , array(
				'method' => 'GET',
				'timeout' => 30,
				'redirection' => 5,
				'httpversion' => '1.0',
				'blocking' => true,
				'headers' => array('Content-type' => 'application/json')
				)
			);
			$body = json_decode( $response['body'] );
			foreach ($body->energy->values as $_values) {
				$sql_insert_production .= "('$data[0]', '$data[1]', '$_values->date', '$_values->value'),";
				$wp_solar_production = $wp_solar_production + $_values->value;
			}
		}
		$sql_insert_sites = rtrim($sql_insert_sites, ",");
		$sql_insert_sites .= ";";
		$sql_insert_production = rtrim($sql_insert_production, ",");
		$sql_insert_production .= ";";
		$wpdb->query('START TRANSACTION;');
		$wpdb->query($sql_drop_sites);
		$wpdb->query($sql_create_sites);
		$wpdb->query($sql_alter_sites);
		$wpdb->query($sql_insert_sites);
//		$wpdb->insert($sites_table_name, $sql_insert_sites);
		$wpdb->query($sql_drop_production);
		$wpdb->query($sql_create_production);
		$wpdb->query($sql_alter_production);
		$wpdb->query($sql_insert_production);
		$wpdb->query('COMMIT;');
		
		set_transient( 'wp_solar_production', $wp_solar_production, DAY_IN_SECONDS );
		$now = time();
		set_transient( 'wp_solar_update', $now, DAY_IN_SECONDS );
		
		require_once ('include/graphs.php');
		wp_solar_power_graph();
		wp_solar_annual_power_graph();
		wp_solar_annual_power_graph_per_site();
	}
}